<?php

App::uses('Component', 'Controller');
App::uses('GoogleCustomSearch', 'EvGoogleSearch.Lib');

class GoogleSearchComponent extends Component {

	protected $_controller = null;

	protected $_apiKey = null;

	protected $_searchEngineId = null;

/**
 * Sets a reference to the calling controller from within the component.
 *
 * @see Component::initialize()
 */
	public function initialize(Controller $controller) {
		parent::initialize($controller);

		$this->_controller = $controller;

		if (empty($this->_apiKey)) {
			$this->_apiKey = Configure::read('EvGoogleSearch.api_key');
		}

		if (empty($this->_searchEngineId)) {
			$this->_searchEngineId = Configure::read('EvGoogleSearch.search_engine_id');
		}
	}

/**
 * Gets the current page number from the querystring
 * @return Int
 */
	public function getPageNumber() {
		if (empty($this->_controller->request->params['named']['page'])) {
			$page = 1;
		} else {
			$page = $this->_controller->request->params['named']['page'];
		}

		return $page;
	}

/**
 * Wrapper method for GoogleCustomSearch class - instanciates class, performs the search and returns an array result with keys:
 *   page
 *   start
 *   end
 *   total_results
 *   results
 *     title
 *     snippet
 *     htmlSnippet
 *     link
 *     image
 *     thumbnail
 *   total_pages
 *   pagination
 *   data - this will only be available if fetch_model_data is set to true in the config
 * @param string terms The search terms
 * @param integer page The page to return
 * @param integer perPage How many results to dispaly per page
 * @param array extra Extra parameters to pass to Google - https://developers.google.com/custom-search/json-api/v1/reference/cse/list
 * @return object The results of the search
 * @throws Exception If error is returned from Google
 */
	public function search($terms, $page = 1, $perPage = 9, $extra = []) {
		$search = new GoogleCustomSearch($this->_searchEngineId, $this->_apiKey);
		$results = $search->search($terms, $page, $perPage, $extra);

		if (!empty($results)) {
			// convert into arrays like we all love
			$results = json_decode(json_encode($results), true);
		}

		$results = $this->_injectImages($results);
		$results = $this->_injectPagination($results, $page, $perPage);

		return $results;
	}

/**
 * Set api key on the fly for cases of multiple custom search projects
 * @param $key
 */
	public function setApiKey($key) {
		$this->_apiKey = $key;
	}

/**
 * Set search engine id on the fly for cases of multiple custom search projects
 * @param $id
 */
	public function setSearchEngineId($id) {
		$this->_searchEngineId = $id;
	}

/**
 * Replaces the google results images with the original by extracting the id from the url and performing a findById on the image model
 * @param  Array $results
 * @return Array $results - with modified image key
 */
	protected function _injectImages($results = null) {
		if (!$results) {
			return false;
		}

		$Image = ClassRegistry::init('EvCore.Image');

		foreach ($results['results'] as &$result) {
			if (!empty($result['image'])) {
				// regex to grab image id from url
				preg_match('#\/img_cache\/(\w*)\/#i', $result['image'], $matches);

				// if we manage to get the image id, we can fetch the original..
				if (!empty($matches[1])) {
					$imageId = $matches[1];

					$image = $Image->findById($imageId);

					// unable to find the image.. (i.e. we might be searching another site)
					if (empty($image[$Image->alias])) {
						continue;
					}

					// replace image fetched from google search with filepath for the original
					$result['image'] = $image[$Image->alias];

					// if set to do so in the config, try and get the data associated with the result from google
					if (Configure::read('EvGoogleSearch.fetch_model_data') == true) {
						$result = $this->_fetchModelData($result);
					}
				}

			}
		}

		return $results;
	}

/**
 * 1) inserts total_pages array key
 * 2) inserts an array key containing an array of page numbers that should display for the current result set
 * @param  $results
 * @param  integer $pageNumber
 * @return Array or false if no results are passed in
 */
	protected function _injectPagination($results = null, $pageNumber = null, $perPage = null) {
		if (empty($results) || empty($pageNumber) || empty($perPage)) {
			return false;
		}

		$results['total_pages'] = ceil($results['total_results'] / $perPage);

		for ($i = 1; $i <= $results['total_pages']; $i++) {
			if (($i < $pageNumber - 5 || $i > $pageNumber + 5) && $results['total_pages'] > 10 && $pageNumber > 4) {
				continue;
			} elseif ($pageNumber < 5 && $i >= 10) {
				continue;
			}

			$results['pagination'][] = $i;
		}

		return $results;
	}

/**
 * fetches model data for a single google search result and inserts it into the results array
 * @param  $result
 * @return Array $result
 */
	protected function _fetchModelData($result) {
		$modelName = $result['image']['model'];
		$modelId = $result['image']['model_id'];

		// load the model thats associated with the image
		$Model = ClassRegistry::init($modelName);

		if (!empty($Model) && !empty($modelId)) {
			// find the data
			$data = $Model->find('first', array('conditions' => 'id = ' . $modelId));

			$result['data'] = $data[$Model->alias];
		}

		return $result;
	}
}
